package types

/*
	Интерфейс к ангару
*/

// IAngar -- интерфейс ангара
type IAngar interface {
	ISection
	// Run -- запускает ангар в работу
	Run() error
	// Convoy -- возвращает объект конвоя
	Convoy() IConvoy
	// Gold -- возвращает объект золота
	Gold() IStatParam
	// Fuel -- возвращает объект топлива
	Fuel() IStatParam
	// Level -- возвращает объект уровня игрока
	Level() IStatParam
	// Progress -- возвращает прогресс уровня игрока
	Progress() IStatParam
	// Online -- возвращает число игроков онлайн
	Online() IStatParam
	// SilverOnline -- возвращает заработанное серебро с момента запуска бота
	SilverOnline() IStatParam
	// SilverAll -- возвращает всё серебро бота
	SilverAll() IStatParam
	// SilverUpdate -- на основе фактического серебра -- обновляет вырабатанное серебро
	SilverUpdate(silverFact int)
	// Battle -- возвращает объект сражения
	Battle() IBattle
	// Base -- возвращает объект базы
	Base() IBase
	// Missions -- возвращает объект миссий
	Missions() IMissions
}
